from __future__ import annotations

from collections import UserDict
from contextvars import ContextVar, Token
from typing import Any, Iterable, final

from .base import UnionPattern
from .core import BasePattern
from .util import AllParam, Empty


@final
class Patterns(UserDict):
    def __init__(self, name: str):
        self.name = name
        super().__init__({"": Empty, "*": AllParam})

    def set(
        self,
        target: BasePattern,
        alias: str | None = None,
        cover: bool = True,
    ):
        """
        增加可使用的类型转换器

        Args:
            target: 设置的表达式
            alias: 目标类型的别名
            cover: 是否覆盖已有的转换器
        """
        for k in {alias, target.alias, target.origin}:
            if not k:
                continue
            if k not in self.data or cover:
                self.data[k] = target
            else:
                al_pat = self.data[k]
                self.data[k] = (
                    UnionPattern([*al_pat.base, target])
                    if isinstance(al_pat, UnionPattern)
                    else (UnionPattern([al_pat, target]))
                )

    def sets(self, patterns: Iterable[BasePattern], cover: bool = True):
        for pat in patterns:
            self.set(pat, cover=cover)

    def merge(self, patterns: dict[str, BasePattern]):
        for k in patterns:
            self.set(patterns[k], alias=k)

    def remove(self, origin_type: type, alias: str | None = None):
        if alias and (al_pat := self.data.get(alias)):
            if isinstance(al_pat, UnionPattern):
                self.data[alias] = UnionPattern(filter(lambda x: x.alias != alias, al_pat.base))  # type: ignore
                if not self.data[alias].base:  # type: ignore # pragma: no cover
                    del self.data[alias]
            else:
                del self.data[alias]
        elif al_pat := self.data.get(origin_type):
            if isinstance(al_pat, UnionPattern):
                self.data[origin_type] = UnionPattern(
                    filter(lambda x: x.origin != origin_type, al_pat.for_validate)
                )
                if not self.data[origin_type].base:  # type: ignore # pragma: no cover
                    del self.data[origin_type]
            else:
                del self.data[origin_type]


_ctx: dict[str, Patterns] = {"$global": Patterns("$global")}
_ctx_token: Token

pattern_ctx: ContextVar[Patterns] = ContextVar("nepatterns")
_ctx_token = pattern_ctx.set(_ctx["$global"])


def create_local_patterns(
    name: str,
    data: dict[Any, BasePattern | type[Empty]] | None = None,
    set_current: bool = True,
) -> Patterns:
    """
    新建一个本地表达式组

    Args:
        name: 组名
        data: 可选的初始内容
        set_current: 是否设置为 current
    """
    global _ctx_token
    if name.startswith("$"):
        raise ValueError(name)
    new = Patterns(name)
    new.update(data or {})
    _ctx[name] = new
    if set_current:
        pattern_ctx.reset(_ctx_token)
        _ctx_token = pattern_ctx.set(new)
    return new


def switch_local_patterns(name: str):
    global _ctx_token
    if name.startswith("$"):
        raise ValueError(name)
    target = _ctx[name]
    pattern_ctx.reset(_ctx_token)
    _ctx_token = pattern_ctx.set(target)


def reset_local_patterns():
    global _ctx_token

    target = _ctx["$global"]
    pattern_ctx.reset(_ctx_token)
    _ctx_token = pattern_ctx.set(target)


def local_patterns():
    local = pattern_ctx.get()
    return local if local.name != "$global" else Patterns("$temp")


def global_patterns():
    return _ctx["$global"]


def all_patterns():
    """获取 global 与 local 的合并表达式组"""
    new = Patterns("$temp")
    local = local_patterns()
    if not local.name.startswith("$"):
        new.update(local_patterns().data)
    new.update(global_patterns().data)
    return new


__all__ = [
    "Patterns",
    "local_patterns",
    "global_patterns",
    "all_patterns",
    "switch_local_patterns",
    "create_local_patterns",
    "reset_local_patterns",
]



from .base import (
    MappingPattern,
    RegexPattern,
    SequencePattern,
    SwitchPattern,
    UnionPattern,
)
from .config import lang
from .context import (
    Patterns,
    all_patterns,
    create_local_patterns,
    local_patterns,
    switch_local_patterns,
    reset_local_patterns
)
from .core import BasePattern, MatchMode, ValidateResult, set_unit
from .exception import MatchFailed
from .main import (
    DATETIME,
    EMAIL,
    FLOAT,
    HEX,
    HEX_COLOR,
    INTEGER,
    IP,
    NUMBER,
    URL,
    AnyOne,
    AnyString,
    Bind,
    type_parser,
)
from .util import AllParam, Empty, TPattern, generic_isinstance



from __future__ import annotations

import inspect
import sys
import types
from datetime import datetime
from copy import deepcopy
from collections.abc import (
    Sequence as ABCSeq,
    Set as ABCSet,
    MutableSet as ABCMuSet,
    MutableSequence as ABCMuSeq,
    MutableMapping as ABCMuMap,
    Mapping as ABCMap,
)
from contextlib import suppress
from functools import lru_cache
from pathlib import Path
from types import FunctionType, LambdaType, MethodType
from typing import Any, Union, Literal, TypeVar, runtime_checkable

try:
    from typing import Annotated, get_args, get_origin  # type: ignore
except ImportError:
    from typing_extensions import Annotated, get_args, get_origin


from .config import lang
from .context import global_patterns, all_patterns
from .core import BasePattern, MatchMode
from .base import UnionPattern, MappingPattern, SequencePattern, RegexPattern, SwitchPattern
from .util import AllParam, Empty, GenericAlias

_Contents = (Union, types.UnionType, Literal) if sys.version_info >= (3, 10) else (Union, Literal)  # pragma: no cover


AnyOne = BasePattern(r".+", MatchMode.KEEP, Any, alias="any")
"""匹配任意内容的表达式"""

AnyString = BasePattern(r".+", MatchMode.TYPE_CONVERT, str, alias="any_str")
"""匹配任意内容并转为字符串的表达式"""

_String = BasePattern(r".+", MatchMode.KEEP, str, alias="str", accepts=[str])

EMAIL = BasePattern(r"(?:[\w\.+-]+)@(?:[\w\.-]+)\.(?:[\w\.-]+)", alias="email")
"""匹配邮箱地址的表达式"""

IP = BasePattern(
    r"(?:(?:[01]{0,1}\d{0,1}\d|2[0-4]\d|25[0-5])\.){3}(?:[01]{0,1}\d{0,1}\d|2[0-4]\d|25[0-5]):?(?:\d+)?",
    alias="ip",
)
"""匹配Ip地址的表达式"""

URL = BasePattern(
    r"(?:[\w]+://)?[^/\s?#]+[^\s?#]+(?:\?[^\s#]*)?(?:#[^\s]*)?", alias="url"
)
"""匹配网页链接的表达式"""

HEX = BasePattern(
    r"((?:0x)?[0-9a-fA-F]+)",
    MatchMode.REGEX_CONVERT,
    int,
    lambda _, x: int(x, 16),
    "hex",
)
"""匹配16进制数的表达式"""

HEX_COLOR = BasePattern(
    r"(#[0-9a-fA-F]{6})", MatchMode.REGEX_CONVERT, str, lambda _, x: x[1:], "color"
)
"""匹配16进制颜色代码的表达式"""

DATETIME = BasePattern(
    model=MatchMode.TYPE_CONVERT,
    origin=datetime,
    alias="datetime",
    accepts=[str, int],
    converter=lambda _, x: datetime.fromtimestamp(x)
    if isinstance(x, int)
    else datetime.fromisoformat(x),
)
"""匹配时间的表达式"""

global_patterns().update(
    {
        Any: AnyOne,
        Ellipsis: AnyOne,
        object: AnyOne,
        "any": AnyOne,
        "any_str": AnyString,
        "email": EMAIL,
        "color": HEX_COLOR,
        "hex": HEX,
        "ip": IP,
        "url": URL,
        "...": AnyOne,
        "datetime": DATETIME,
    }
)

StrPath = BasePattern(
    model=MatchMode.TYPE_CONVERT, origin=Path, alias="path", accepts=[str]
)
PathFile = BasePattern(
    model=MatchMode.TYPE_CONVERT,
    origin=bytes,
    alias="file",
    accepts=[Path],
    previous=StrPath,
    converter=lambda _, x: x.read_bytes() if x.exists() and x.is_file() else None,  # type: ignore
)

INTEGER = BasePattern(
    r"(\-?\d+)", MatchMode.REGEX_CONVERT, int, lambda _, x: int(x), "int"
)
"""整形数表达式，只能接受整数样式的量"""

FLOAT = BasePattern(
    r"(\-?\d+\.?\d*)", MatchMode.REGEX_CONVERT, float, lambda _, x: float(x), "float"
)
"""浮点数表达式"""

NUMBER = BasePattern(
    r"(\-?\d+\.?\d*)",
    MatchMode.TYPE_CONVERT,
    int,
    lambda _, x: int(float(x)),
    "number",
    accepts=[FLOAT, int],
)
"""一般数表达式，既可以浮点数也可以整数"""

_Bool = BasePattern(
    r"(?i:True|False)",
    MatchMode.REGEX_CONVERT,
    bool,
    lambda _, x: x.lower() == "true",
    "bool",
)
_List = BasePattern(r"(\[.+?\])", MatchMode.REGEX_CONVERT, list, alias="list")
_Tuple = BasePattern(r"(\(.+?\))", MatchMode.REGEX_CONVERT, tuple, alias="tuple")
_Set = BasePattern(r"(\{.+?\})", MatchMode.REGEX_CONVERT, set, alias="set")
_Dict = BasePattern(r"(\{.+?\})", MatchMode.REGEX_CONVERT, dict, alias="dict")

global_patterns().sets([PathFile, _String, INTEGER, FLOAT, _Bool, _List, _Tuple, _Set, _Dict])
global_patterns()["number"] = NUMBER


def _generic_parser(item: GenericAlias, extra: str) -> BasePattern:
    origin = get_origin(item)
    if origin is Annotated:
        org, *meta = get_args(item)
        if not isinstance(_o := type_parser(org, extra), BasePattern):  # type: ignore  # pragma: no cover
            raise TypeError(_o)
        _arg = deepcopy(_o)
        _arg.alias = (
            al[-1] if (al := [i for i in meta if isinstance(i, str)]) else _arg.alias
        )
        _arg.validators.extend(i for i in meta if callable(i))
        return _arg
    if origin in _Contents:
        _args = {type_parser(t, extra) for t in get_args(item)}
        return (_args.pop() if len(_args) == 1 else UnionPattern(_args)) if _args else AnyOne
    if origin in (dict, ABCMap, ABCMuMap):
        if args := get_args(item):
            return MappingPattern(
                arg_key=type_parser(args[0], "ignore"),
                arg_value=type_parser(args[1], "allow"),
            )
        return MappingPattern(AnyOne, AnyOne)  # pragma: no cover
    _args = type_parser(args[0], "allow") if (args := get_args(item)) else AnyOne
    if origin in (ABCMuSeq, list):
        return SequencePattern(list, _args)
    if origin in (ABCSeq, tuple):
        return SequencePattern(tuple, _args)
    if origin in (ABCMuSet, ABCSet, set):
        return SequencePattern(set, _args)
    return BasePattern("", 0, origin, alias=f"{repr(item).split('.')[-1]}", accepts=[origin])  # type: ignore


def _typevar_parser(item: TypeVar):
    return BasePattern(model=MatchMode.KEEP, origin=Any, alias=f'{item}'[1:], accepts=[item])  # type: ignore


def _protocol_parser(item: type):
    if not getattr(item, '_is_runtime_protocol', True):  # pragma: no cover
        item = runtime_checkable(deepcopy(item))  # type: ignore
    return BasePattern(model=MatchMode.KEEP, origin=Any, alias=f'{item}', accepts=[item])


def type_parser(item: Any, extra: str = "allow") -> BasePattern:
    """对数类型的检查， 将一般数据类型转为 BasePattern 或者特殊类型"""
    if isinstance(item, BasePattern) or item is AllParam:
        return item
    with suppress(TypeError):
        if item and (pat := all_patterns().get(item, None)):
            return pat
    with suppress(TypeError):
        if not inspect.isclass(item) and isinstance(item, GenericAlias):
            return _generic_parser(item, extra)
    if isinstance(item, TypeVar):
        return _typevar_parser(item)
    if inspect.isclass(item) and getattr(item, "_is_protocol", False):
        return _protocol_parser(item)
    if isinstance(item, (FunctionType, MethodType, LambdaType)):
        if len((sig := inspect.signature(item)).parameters) not in (1, 2):  # pragma: no cover
            raise TypeError(f"{item} can only accept 1 or 2 argument")
        anno = list(sig.parameters.values())[-1].annotation
        return BasePattern(
            accepts=[]
            if anno == Empty
            else list(_)
            if (_ := get_args(anno))
            else [anno],
            origin=Any if sig.return_annotation == Empty else sig.return_annotation,
            converter=item if len(sig.parameters) == 2 else lambda _, x: item(x),
            model=MatchMode.TYPE_CONVERT,
        )
    if isinstance(item, str):
        if item.startswith("re:"):
            return RegexPattern(item[3:])
        if "|" in item:
            names = item.split("|")
            return UnionPattern(all_patterns().get(i, i) for i in names if i)
        return BasePattern(item, alias=f"'{item}'")
    if isinstance(
        item, (list, tuple, set, ABCSeq, ABCMuSeq, ABCSet, ABCMuSet)
    ):  # Args[foo, [123, int]]
        return UnionPattern(
            map(lambda x: type_parser(x) if inspect.isclass(x) else x, item)
        )
    if isinstance(item, (dict, ABCMap, ABCMuMap)):
        return SwitchPattern(dict(item))
    if item is None or type(None) == item:
        return Empty  # type: ignore
    if extra == "ignore":
        return AnyOne
    elif extra == "reject":
        raise TypeError(lang.validate_reject.format(target=item))
    return BasePattern.of(item) if inspect.isclass(item) else BasePattern.on(item)


class Bind:
    __slots__ = ()

    def __new__(cls, *args, **kwargs):  # pragma: no cover
        raise TypeError("Type Bind cannot be instantiated.")

    @classmethod
    @lru_cache(maxsize=None)
    def __class_getitem__(cls, params) -> BasePattern:
        if not isinstance(params, tuple) or len(params) < 2:
            raise TypeError(
                "Bind[...] should be used with only two arguments (a type and an annotation)."
            )
        if not (
            pattern := params[0]
            if isinstance(params[0], BasePattern)
            else all_patterns().get(params[0])
        ):
            raise ValueError("Bind[...] first argument should be a BasePattern.")
        if not all(callable(i) or isinstance(i, str) for i in params[1:]):
            raise TypeError("Bind[...] second argument should be a callable or str.")
        pattern = deepcopy(pattern)
        pattern.alias = (
            al[0]
            if (al := [i for i in params[1:] if isinstance(i, str)])
            else pattern.alias
        )
        pattern.validators.extend(filter(callable, params[1:]))
        return pattern


__all__ = [
    "Bind",
    "AnyString",
    "type_parser",
    "AnyOne",
    "StrPath",
    "PathFile",
    "NUMBER",
    "HEX",
    "HEX_COLOR",
    "IP",
    "URL",
    "EMAIL",
    "DATETIME",
    "INTEGER",
    "FLOAT",
]



from __future__ import annotations

import re
from enum import IntEnum, Enum
from copy import deepcopy
from typing import (
    TypeVar,
    Callable,
    Any,
    Generic,
    overload,
)
from dataclasses import dataclass, field

try:
    from typing import Annotated, Self, get_origin  # type: ignore
except ImportError:
    from typing_extensions import Annotated, Self, get_origin

from .exception import MatchFailed
from .config import lang
from .util import generic_isinstance, TPattern, Empty


def _accept(
    input_: Any,
    patterns: list[BasePattern] | None = None,
    types: list[type] | None = None,
):
    res_p = any(map(lambda x: x(input_).success, patterns)) if patterns else False
    res_t = generic_isinstance(input_, tuple(types)) if types else False
    return res_t or res_p


class MatchMode(IntEnum):
    """参数表达式匹配模式"""

    REGEX_CONVERT = 3
    """正则匹配并转换"""
    TYPE_CONVERT = 2
    """传入值直接转换"""
    REGEX_MATCH = 1
    """正则匹配"""
    KEEP = 0
    """保持传入值"""


class ResultFlag(str, Enum):
    """参数表达式验证结果标识符"""

    VALID = "valid"
    "验证通过"
    ERROR = "error"
    "验证失败"
    DEFAULT = "default"
    "默认值"


T = TypeVar("T")
TOrigin = TypeVar("TOrigin")
TVOrigin = TypeVar("TVOrigin")
TDefault = TypeVar("TDefault")


@dataclass
class ValidateResult(Generic[TVOrigin]):
    _value: TVOrigin | type[Empty] = field(default=Empty)
    _error: Exception | type[Empty] = field(default=Empty)
    flag: ResultFlag = field(default=ResultFlag.VALID)

    @property
    def value(self) -> TVOrigin:
        if self.flag == ResultFlag.ERROR or self._value == Empty:
            raise RuntimeError("cannot access value")
        return self._value  # type: ignore


    @property
    def error(self) -> Exception | None:
        if self.flag == ResultFlag.ERROR and self._error != Empty:
            assert isinstance(self._error, Exception)
            return self._error

    @property
    def success(self) -> bool:
        return self.flag == ResultFlag.VALID

    @property
    def failed(self) -> bool:
        return self.flag == ResultFlag.ERROR

    @property
    def or_default(self) -> bool:
        return self.flag == ResultFlag.DEFAULT

    @overload
    def step(self, other: type[T] | T) -> T:
        ...

    @overload
    def step(self, other: Callable[[TVOrigin], T]) -> T | Self:
        ...

    @overload
    def step(self, other: BasePattern[T]) -> ValidateResult[T]:
        ...

    def step(
        self, other: type[T] | Callable[[TVOrigin], T] | Any | BasePattern[T]
    ) -> T | Self | ValidateResult[T]:
        if other is bool:
            return self.success  # type: ignore
        if callable(other) and self.success:
            return other(self.value)
        if self.success and hasattr(self.value, "__rshift__"):
            return self.value | other  # type: ignore
        return self

    @overload
    def __rshift__(self, other: type[T]| T) -> T:
        ...

    @overload
    def __rshift__(self, other: Callable[[TVOrigin], T]) -> T | Self:
        ...

    @overload
    def __rshift__(self, other: BasePattern[T]) -> ValidateResult[T]:
        ...

    def __rshift__(
        self, other: type[T] | Callable[[TVOrigin], T] | Any
    ) -> T | Self | ValidateResult[T]:
        return self.step(other)  # type: ignore

    def __bool__(self):
        return self.success


class BasePattern(Generic[TOrigin]):
    """对参数类型值的包装"""

    regex_pattern: TPattern  # type: ignore
    pattern: str
    mode: MatchMode
    converter: Callable[[BasePattern[TOrigin], str | Any], TOrigin]
    validators: list[Callable[[TOrigin], bool]]

    anti: bool
    origin: type[TOrigin]
    pattern_accepts: list[BasePattern]
    type_accepts: list[type]
    alias: str | None
    previous: BasePattern | None

    __slots__ = (
        "regex_pattern",
        "pattern",
        "mode",
        "converter",
        "anti",
        "origin",
        "pattern_accepts",
        "type_accepts",
        "alias",
        "previous",
        "validators",
    )

    def __init__(
        self,
        pattern: str = ".+",
        model: int | MatchMode = MatchMode.REGEX_MATCH,
        origin: type[TOrigin] = str,
        converter: Callable[[BasePattern[TOrigin], str | Any], TOrigin] | None = None,
        alias: str | None = None,
        previous: BasePattern | None = None,
        accepts: list[type | BasePattern] | None = None,
        validators: list[Callable[[TOrigin], bool]] | None = None,
        anti: bool = False,
    ):
        """
        初始化参数匹配表达式
        """
        if pattern.startswith("^") or pattern.endswith("$"):
            raise ValueError(lang.pattern_head_or_tail_error.format(target=pattern))
        self.pattern = pattern
        self.regex_pattern = re.compile(f"^{pattern}$")
        self.mode = MatchMode(model)
        self.origin = origin
        self.alias = alias
        self.previous = previous
        accepts = accepts or []
        self.pattern_accepts = list(
            filter(lambda x: isinstance(x, BasePattern), accepts)  # type: ignore
        )
        self.type_accepts = list(
            filter(lambda x: not isinstance(x, BasePattern), accepts)  # type: ignore
        )
        self.converter = converter or (
            lambda _, x: (get_origin(origin) or origin)(x)
            if model == MatchMode.TYPE_CONVERT
            else eval(x)
        )
        self.validators = validators or []
        self.anti = anti

    def __repr__(self):
        if self.mode == MatchMode.KEEP:
            if self.alias:
                return self.alias
            return (
                "Any" if not self.type_accepts and not self.pattern_accepts else
                "|".join([x.__name__ for x in self.type_accepts] + [x.__repr__() for x in self.pattern_accepts])
            )

        if not self.alias:
            name = getattr(self.origin, "__name__", str(self.origin))
            if self.mode == MatchMode.REGEX_MATCH:
                text = self.pattern
            elif self.mode == MatchMode.REGEX_CONVERT or (
                not self.type_accepts and not self.pattern_accepts
            ):
                text = name
            else:
                text = (
                    "|".join(
                        [x.__name__ for x in self.type_accepts]
                        + [x.__repr__() for x in self.pattern_accepts]
                    )
                    + f" -> {name}"
                )
        else:
            text = self.alias
        return (
            f"{f'{self.previous.__repr__()} -> ' if self.previous and id(self.previous) != id(self) else ''}"
            f"{'!' if self.anti else ''}{text}"
        )

    def __str__(self):
        return self.__repr__()

    def __hash__(self):
        return hash(self.__repr__())

    def __eq__(self, other):
        return isinstance(other, BasePattern) and self.__repr__() == other.__repr__()

    @staticmethod
    def of(unit: type[TOrigin]) -> BasePattern[TOrigin]:
        """提供 Type[DataUnit] 类型的构造方法"""
        return BasePattern(
            "", MatchMode.KEEP, unit, alias=unit.__name__, accepts=[unit]
        )

    @staticmethod
    def on(obj: TOrigin) -> BasePattern[TOrigin]:
        """提供 DataUnit 类型的构造方法"""
        return BasePattern(
            "",
            MatchMode.KEEP,
            type(obj),
            alias=str(obj),
            validators=[lambda x: x == obj],
        )

    @staticmethod
    def to(content: Any) -> BasePattern | None:
        """便捷的使用 type_parser 的方法"""
        from .main import type_parser

        if isinstance(res := type_parser(content, "allow"), BasePattern):
            return res

    def reverse(self) -> Self:
        """改变 pattern 的 anti 值"""
        self.anti = not self.anti
        return self

    def prefixed(self):
        """让表达式能在某些场景下实现前缀匹配; 返回自身的拷贝"""
        cp_self = deepcopy(self)
        if self.mode in (MatchMode.REGEX_MATCH, MatchMode.REGEX_CONVERT):
            cp_self.regex_pattern = re.compile(f"^{self.pattern}")
        return cp_self

    def suffixed(self):
        """让表达式能在某些场景下实现后缀匹配; 返回自身的拷贝"""
        cp_self = deepcopy(self)
        if self.mode in (MatchMode.REGEX_MATCH, MatchMode.REGEX_CONVERT):
            cp_self.regex_pattern = re.compile(f".*?({self.pattern})$")
        return cp_self

    def match(self, input_: str | Any) -> TOrigin:
        """
        对传入的参数进行匹配, 如果匹配成功, 则返回转换后的值, 否则返回None
        """
        if (
            self.mode > 0
            and self.origin not in (str, Any)
            and generic_isinstance(input_, self.origin)
        ):
            return input_  # type: ignore
        if (self.type_accepts or self.pattern_accepts) and not _accept(
            input_, self.pattern_accepts, self.type_accepts
        ):
            if not self.previous or not _accept(
                input_ := self.previous.match(input_),
                self.pattern_accepts,
                self.type_accepts,
            ):  # pragma: no cover
                raise MatchFailed(lang.type_error.format(target=input_.__class__))
        if self.mode == MatchMode.KEEP:
            return input_  # type: ignore
        if self.mode == MatchMode.TYPE_CONVERT:
            res = self.converter(self, input_)
            if res is None and self.origin == Any:  # pragma: no cover
                raise MatchFailed(lang.content_error.format(target=input_))
            if not generic_isinstance(res, self.origin):
                if not self.previous or not generic_isinstance(
                    res := self.converter(self, self.previous.match(input_)), self.origin
                ):
                    raise MatchFailed(lang.content_error.format(target=input_))
            return res
        if not isinstance(input_, str):
            if not self.previous or not isinstance(
                input_ := self.previous.match(input_), str
            ):
                raise MatchFailed(lang.type_error.format(target=type(input_)))
        if mat := self.regex_pattern.match(input_):
            glen = len(mat.groups())
            return (
                self.converter(self, mat[1] if glen > 0 else mat[0])
                if self.mode == MatchMode.REGEX_CONVERT
                else mat[1] if glen > 0 else mat[0]
            )
        raise MatchFailed(lang.content_error.format(target=input_))

    @overload
    def validate(self, input_: Any) -> ValidateResult[TOrigin]:
        ...

    @overload
    def validate(self, input_: Any, default: TDefault) -> ValidateResult[TOrigin | TDefault]:
        ...

    def validate(  # type: ignore
        self, input_: Any, default: TDefault | None = None
    ) -> ValidateResult[TOrigin | TDefault]:
        """
        对传入的值进行正向验证，返回可能的匹配与转化结果。

        若传入默认值，验证失败会返回默认值
        """
        try:
            res = self.match(input_)
            for i in self.validators:
                if not i(res):
                    raise MatchFailed(lang.content_error.format(target=input_))
            return ValidateResult(_value=res, flag=ResultFlag.VALID)
        except Exception as e:
            if default is None:
                return ValidateResult(_error=e, flag=ResultFlag.ERROR)
            return ValidateResult(
                _value=None if default is Empty else default, flag=ResultFlag.DEFAULT  # type: ignore
            )

    @overload
    def invalidate(self, input_: Any) -> ValidateResult[Any]:
        ...

    @overload
    def invalidate(self, input_: Any, default: TDefault) -> ValidateResult[Any | TDefault]:
        ...

    def invalidate(
        self, input_: Any, default: TDefault | None = None
    ) -> ValidateResult[Any | TDefault]:
        """
        对传入的值进行反向验证，返回可能的匹配与转化结果。

        若传入默认值，验证失败会返回默认值
        """
        try:
            res = self.match(input_)
        except MatchFailed:
            return ValidateResult(_value=input_, flag=ResultFlag.VALID)
        else:
            for i in self.validators:
                if not i(res):
                    return ValidateResult(_value=input_, flag=ResultFlag.VALID)
            if default is None:
                return ValidateResult(
                    _error=MatchFailed(lang.content_error.format(target=input_)),
                    flag=ResultFlag.ERROR,
                )
            return ValidateResult(
                _value=None if default is Empty else default, flag=ResultFlag.DEFAULT
            )


    @overload
    def __call__(self, input_: Any) -> ValidateResult[TOrigin]:
        ...

    @overload
    def __call__(self, input_: Any, default: TDefault) -> ValidateResult[TOrigin | TDefault]:
        ...
    def __call__(self, input_: Any, default: TDefault | None = None) -> ValidateResult[TOrigin | TDefault | None]:  # type: ignore
        """
        依据 anti 值 自动选择验证方式
        """
        if self.anti:
            return self.invalidate(input_, default)
        else:
            return self.validate(input_, default)

    def __rrshift__(self, other):
        return self.__call__(other)

    def __rmatmul__(self, other) -> Self:  # pragma: no cover
        if isinstance(other, str):
            self.alias = other
        return self

    def __matmul__(self, other) -> Self:  # pragma: no cover
        if isinstance(other, str):
            self.alias = other
        return self


def set_unit(
    target: type[TOrigin], predicate: Callable[..., bool]
) -> Annotated[TOrigin, ...]:
    """通过predicate区分同一个类的不同情况"""
    return Annotated[target, predicate]  # type: ignore


__all__ = ["MatchMode", "BasePattern", "set_unit", "ValidateResult", "TOrigin"]
